@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2006, 2007, 2008, 2009, 2010 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2002, 2006-2012 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -38,10 +36,10 @@ imap4d_login (struct imap4d_command *command, imap4d_tokbuf_t tok)
   int rc;
 
   if (login_disabled || tls_required)    
-    return util_finish (command, RESP_NO, "Command disabled");
+    return io_completion_response (command, RESP_NO, "Command disabled");
 
   if (imap4d_tokbuf_argc (tok) != 4)
-    return util_finish (command, RESP_BAD, "Invalid arguments");
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
   
   username = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
   pass = imap4d_tokbuf_getarg (tok, IMAP4_ARG_2);
@@ -51,7 +49,8 @@ imap4d_login (struct imap4d_command *command, imap4d_tokbuf_t tok)
   if (auth_data == NULL)
     {
       mu_diag_output (MU_DIAG_INFO, _("user `%s' nonexistent"), username);
-      return util_finish (command, RESP_NO, "User name or passwd rejected");
+      return io_completion_response (command, RESP_NO, 
+                                     "User name or passwd rejected");
     }
 
   rc = mu_authenticate (auth_data, pass);
@@ -59,11 +58,13 @@ imap4d_login (struct imap4d_command *command, imap4d_tokbuf_t tok)
   if (rc)
     {
       mu_diag_output (MU_DIAG_INFO, _("login failed: %s"), username);
-      return util_finish (command, RESP_NO, "User name or passwd rejected");
+      return io_completion_response (command, RESP_NO, 
+                                     "User name or passwd rejected");
     }
 
   if (imap4d_session_setup0 ())
-    return util_finish (command, RESP_NO, "User name or passwd rejected");
-  return util_finish (command, RESP_OK, "Completed");
+    return io_completion_response (command, RESP_NO, 
+                                   "User name or passwd rejected");
+  return io_completion_response (command, RESP_OK, "Completed");
 }
 
