@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
-   Software Foundation, Inc.
+   Copyright (C) 2003, 2005-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -303,7 +302,7 @@ check_file (char *name)
       mu_error (_("cannot create message list"));
       return 1;
     }
-  elt = xmalloc (sizeof *elt);
+  elt = mu_alloc (sizeof *elt);
   elt->file_name = file_name;
   elt->msg = msg;
   return mu_list_append (mesg_list, elt);
@@ -327,7 +326,7 @@ read_mts_profile ()
       name = mh_expand_name (MHLIBDIR, "mtstailor", 0);
       mts_profile = mh_read_property_file (name, 1);
 
-      name = mu_tilde_expansion ("~/.mtstailor", "/", NULL);
+      name = mu_tilde_expansion ("~/.mtstailor", MU_HIERARCHY_DELIMITER, NULL);
       local_profile = mh_read_property_file (name, 1);
 
       mh_property_merge (mts_profile, local_profile);
@@ -355,7 +354,7 @@ read_mts_profile ()
     {
       char *newdomain;
 
-      newdomain = xmalloc (strlen (hostname) + 1 + strlen (p) + 1);
+      newdomain = mu_alloc (strlen (hostname) + 1 + strlen (p) + 1);
       strcpy (newdomain, hostname);
       strcat (newdomain, ".");
       strcat (newdomain, p);
@@ -389,7 +388,7 @@ read_mts_profile ()
 	  exit (1);
 	}
       len = strlen (p) + 1 + strlen (domain) + 1;
-      newemail = xmalloc (len);
+      newemail = mu_alloc (len);
       strcpy (newemail, p);
       strcat (newemail, "@");
       strcat (newemail, domain);
@@ -475,11 +474,10 @@ get_sender_personal ()
 static void
 set_address_header (mu_header_t hdr, char *name, mu_address_t addr)
 {
-  size_t s = mu_address_format_string (addr, NULL, 0);
-  char *value = xmalloc (s + 1);
-  mu_address_format_string (addr, value, s);
-  mu_header_set_value (hdr, name, value, 1);
-  free (value);
+  const char *value;
+  if (mu_address_sget_printable (addr, &value) == 0)
+    mu_header_set_value (hdr, name, value, 1);
+  /* FIXME: Error reporting */
 }
 
 void
@@ -623,7 +621,7 @@ fix_dcc (mu_message_t msg)
 void
 backup_file (const char *file_name)
 {
-  char *new_name = xmalloc (strlen (file_name) + 2);
+  char *new_name = mu_alloc (strlen (file_name) + 2);
   char *p = strrchr (file_name, '/');
   if (p)
     {
@@ -738,6 +736,27 @@ _action_send (void *item, void *data)
   return 0;
 }
 
+static int
+_add_to_mesg_list (size_t num, mu_message_t msg, void *data)
+{
+  char const *path = data;
+  struct list_elt *elt;
+  size_t uid;
+  int rc;
+  
+  elt = mu_alloc (sizeof *elt);
+  elt->msg = msg;
+  mu_message_get_uid (msg, &uid);
+  elt->file_name = mu_make_file_name (path, mu_umaxtostr (0, uid));
+  rc = mu_list_append (mesg_list, elt);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_append", NULL, rc);
+      exit (1);
+    }
+  return 0;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -753,19 +772,20 @@ main (int argc, char **argv)
 		 opt_handler, NULL, &index);
 
   mh_read_aliases ();
-  
+  /* Process the mtstailor file */
+  read_mts_profile ();
+ 
   argc -= index;
   argv += index;
 
   if (draftfolder)
     {
-      mh_msgset_t msgset;
+      mu_msgset_t msgset;
       mu_url_t url;
       const char *path;
-      size_t i;
       
       mbox = mh_open_folder (draftfolder, MU_STREAM_RDWR|MU_STREAM_CREAT);
-      mh_msgset_parse (mbox, &msgset, argc, argv, draftmessage);
+      mh_msgset_parse (&msgset, mbox, argc, argv, draftmessage);
       mu_mailbox_get_url (mbox, &url);
       mu_url_sget_path (url, &path);
       if ((rc = mu_list_create (&mesg_list)))
@@ -773,25 +793,9 @@ main (int argc, char **argv)
 	  mu_error (_("cannot create message list: %s"), mu_strerror (rc));
 	  exit (1);
 	}
-      for (i = 0; i < msgset.count; i++)
-	{
-	  struct list_elt *elt;
-	  size_t uid;
-	  
-	  elt = xmalloc (sizeof *elt);
-	  mu_mailbox_get_message (mbox, msgset.list[i], &elt->msg);
-	  mu_message_get_uid (elt->msg, &uid);
-	  elt->file_name =
-	    mu_make_file_name (path, mu_umaxtostr (0, uid));
-	  rc = mu_list_append (mesg_list, elt);
-	  if (rc)
-	    {
-	      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_append", NULL, rc);
-	      exit (1);
-	    }
-	}
+      mu_msgset_foreach_message (msgset, _add_to_mesg_list, (void*)path);
       
-      mh_msgset_free (&msgset);
+      mu_msgset_free (msgset);
     }
   else
     {
@@ -821,10 +825,7 @@ main (int argc, char **argv)
 	  return 1;
     }
 
-  /* Process the mtstailor file and detach from the console if
-     required */
-  read_mts_profile ();
-  
+  /* Detach from the console if required */
   if (background && daemon (0, 0) < 0)
     {
       mu_error (_("cannot switch to background: %s"), mu_strerror (errno));
@@ -838,7 +839,7 @@ main (int argc, char **argv)
   free (p);
   
   /* Finally, do the work */
-  rc = mu_list_do (mesg_list, _action_send, NULL);
+  rc = mu_list_foreach (mesg_list, _action_send, NULL);
 
   mu_mailbox_destroy (&mbox);
   return !!rc;
