@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* MH sortm command */
 
@@ -23,10 +21,9 @@
 #include <unistd.h>
 #include <signal.h>
 
-const char *program_version = "sortm (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH sortm")"\v"
 N_("Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("[msgs]");
+static char args_doc[] = N_("[MSGLIST]");
 
 #define ARG_QUICKSORT 1024
 #define ARG_SHELL     1025
@@ -34,62 +31,59 @@ static char args_doc[] = N_("[msgs]");
 /* GNU options */
 static struct argp_option options[] = {
   {"folder",        ARG_FOLDER,        N_("FOLDER"), 0,
-   N_("Specify folder to operate upon")},
+   N_("specify folder to operate upon")},
 
   {N_("Setting sort keys:"), 0,  NULL, OPTION_DOC,  NULL, 0},
   {"datefield",     ARG_DATEFIELD,     N_("STRING"), 0,
-   N_("Sort on the date field (default `Date:')"), 10},
+   N_("sort on the date field (default `Date:')"), 10},
   {"nodatefield",   ARG_NODATEFIELD,   NULL,       0,
-   N_("Undo the effect of the last --datefield option"), 10},
+   N_("undo the effect of the last --datefield option"), 10},
   {"limit",         ARG_LIMIT,         N_("DAYS"), 0,
-   N_("Consider two datefields equal if their difference lies within the given nuber of DAYS."), 11},
+   N_("consider two datefields equal if their difference lies within the given nuber of DAYS."), 11},
   {"nolimit",       ARG_NOLIMIT,       NULL,       0,
-   N_("Undo the effect of the last --limit option"), 11},
+   N_("undo the effect of the last --limit option"), 11},
   {"textfield",     ARG_TEXTFIELD,     N_("STRING"), 0,
-   N_("Sort on the text field"), 15},
+   N_("sort on the text field"), 15},
   {"notextfield",   ARG_NOTEXTFIELD,   NULL,       0,
-   N_("Undo the effect of the last --textfield option"), 15},
+   N_("undo the effect of the last --textfield option"), 15},
   {"numfield",      ARG_NUMFIELD,      N_("STRING"), 0,
-   N_("Sort on the numeric field"), 16},
+   N_("sort on the numeric field"), 16},
   
   {N_("Actions:"), 0,  NULL, OPTION_DOC,  NULL, 16},
   {"reorder", ARG_REORDER,    0, 0,
-   N_("Reorder the messages (default)"), 20 },
+   N_("reorder the messages (default)"), 20 },
   {"dry-run", ARG_DRY_RUN,    0, 0,
-   N_("Do not do anything, only show what would have been done"), 20 },
+   N_("do not do anything, only show what would have been done"), 20 },
   {"list",    ARG_LIST,   0, 0,
-   N_("List the sorted messages"), 20 },
+   N_("list the sorted messages"), 20 },
   {"form",    ARG_FORM, N_("FILE"),   0,
-   N_("Read format from given file"), 23},
+   N_("read format from given file"), 23},
   {"format",  ARG_FORMAT, N_("FORMAT"), 0,
-   N_("Use this format string"), 23},
+   N_("use this format string"), 23},
 
   {"verbose",       ARG_VERBOSE,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Verbosely list executed actions"), 30 },
+   N_("verbosely list executed actions"), 30 },
   {"noverbose",     ARG_NOVERBOSE,     NULL, OPTION_HIDDEN, "" },
 
   {N_("Select sort algorithm:"), 0,  NULL, OPTION_DOC,  NULL, 30},
 
   {"shell",     ARG_SHELL,      0, 0,
-   N_("Use shell algorithm"), 40 },
+   N_("use shell algorithm"), 40 },
   {"quicksort", ARG_QUICKSORT,  0, 0,
-   N_("Use quicksort algorithm (default)"), 40 },
-
-  {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("use quicksort algorithm (default)"), 40 },
 
   { NULL },
 };
 
 /* Traditional MH options */
 struct mh_option mh_option[] = {
-  {"datefield",     1, 0, "field" },
-  {"nodatefield",   3, 0, 0 },
-  {"textfield",     1, 0, "field" },
-  {"notextfield",   3, 0, 0 },
-  {"limit",         1, 0, "days" },
-  {"nolimit",       3, 0, 0 },
-  {"verbose",       1, MH_OPT_BOOL, NULL},
+  { "datefield",     MH_OPT_ARG, "field" },
+  { "nodatefield" },
+  { "textfield",     MH_OPT_ARG, "field" },
+  { "notextfield" },
+  { "limit",         MH_OPT_ARG, "days" },
+  { "nolimit" },
+  { "verbose",       MH_OPT_BOOL },
   { NULL },
 };
 
@@ -97,7 +91,11 @@ static int limit;
 static int verbose;
 static mu_mailbox_t mbox;
 static const char *mbox_path;
-static mh_msgset_t msgset;
+
+static size_t *msgarr;
+static size_t msgcount;
+
+static size_t current_num;
 
 #define ACTION_REORDER   0
 #define ACTION_DRY_RUN   1
@@ -115,8 +113,8 @@ static int comp_text (void *a, void *b);
 static int comp_date (void *a, void *b);
 static int comp_number (void *a, void *b);
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -153,7 +151,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_VERBOSE:
-      if (!arg || isalpha (arg[0]))
+      if (!arg || mu_isalpha (arg[0]))
 	verbose = is_true (arg);
       else
 	verbose = arg[0] - '0';
@@ -190,19 +188,16 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       algorithm = key;
       break;
       
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
-      break;
-
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
 
 
 /* *********************** Comparison functions **************************** */
-struct comp_op {
+struct comp_op
+{
   char *field;
   compfun comp;
 };
@@ -212,19 +207,24 @@ static mu_list_t oplist;
 static void
 addop (char *field, compfun comp)
 {
-  struct comp_op *op = xmalloc (sizeof (*op));
+  struct comp_op *op = mu_alloc (sizeof (*op));
   
-  if (!oplist && mu_list_create (&oplist))
+  if (!oplist)
     {
-      mu_error (_("can't create operation list"));
-      exit (1);
+      if (mu_list_create (&oplist))
+	{
+	  mu_error (_("can't create operation list"));
+	  exit (1);
+	}
+      mu_list_set_destroy_item (oplist, mu_list_free_item);
     }
   op->field = field;
   op->comp = comp;
   mu_list_append (oplist, op);
 }
 
-struct rem_data {
+struct rem_data
+{
   struct comp_op *op;
   compfun comp;
 };
@@ -245,12 +245,12 @@ remop (compfun comp)
   struct rem_data d;
   d.comp = comp;
   d.op = NULL;
-  mu_list_do (oplist, rem_action, &d);
+  mu_list_foreach (oplist, rem_action, &d);
   mu_list_remove (oplist, d.op);
-  free (d.op);
 }
 
-struct comp_data {
+struct comp_data
+{
   int r;
   mu_message_t m[2];
 };
@@ -276,11 +276,11 @@ compare_action (void *item, void *data)
 
   ap = a;
   bp = b;
-  if (strcasecmp (op->field, MU_HEADER_SUBJECT) == 0)
+  if (mu_c_strcasecmp (op->field, MU_HEADER_SUBJECT) == 0)
     {
-      if (strncasecmp (ap, "re:", 3) == 0)
+      if (mu_c_strncasecmp (ap, "re:", 3) == 0)
 	ap += 3;
-      if (strncasecmp (b, "re:", 3) == 0)
+      if (mu_c_strncasecmp (b, "re:", 3) == 0)
 	bp += 3;
     }
   
@@ -299,7 +299,7 @@ compare_messages (mu_message_t a, mu_message_t b, size_t anum, size_t bnum)
   d.r = 0;
   d.m[0] = a;
   d.m[1] = b;
-  mu_list_do (oplist, compare_action, &d);
+  mu_list_foreach (oplist, compare_action, &d);
   if (d.r == 0)
     {
       if (anum < bnum)
@@ -316,7 +316,7 @@ compare_messages (mu_message_t a, mu_message_t b, size_t anum, size_t bnum)
 static int
 comp_text (void *a, void *b)
 {
-  return strcasecmp (a, b);
+  return mu_c_strcasecmp (a, b);
 }
 
 static int
@@ -338,12 +338,12 @@ static int
 _parse_822_date (char *date, time_t * timep)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   const char *p = date;
 
   if (mu_parse822_date_time (&p, date + strlen (date), &tm, &tz) == 0)
     {
-      *timep = mu_tm2time (&tm, &tz);
+      *timep = mu_datetime_to_utc (&tm, &tz);
       return 0;
     }
   return 1;
@@ -418,17 +418,17 @@ shell_sort ()
     int h, s, i, j;
     size_t hold;
 
-    for (h = startdst (msgset.count, &s); s > 0; s--, h = prevdst (h))
+    for (h = startdst (msgcount, &s); s > 0; s--, h = prevdst (h))
       {
 	if (verbose > 1)
 	  fprintf (stderr, _("distance %d\n"), h);
-        for (j = h; j < msgset.count; j++)
+        for (j = h; j < msgcount; j++)
 	  {
-            hold = msgset.list[j];
+            hold = msgarr[j];
             for (i = j - h;
-		 i >= 0 && comp0 (hold, msgset.list[i]) < 0; i -= h)
-	      msgset.list[i + h] = msgset.list[i];
-	    msgset.list[i + h] = hold;
+		 i >= 0 && comp0 (hold, msgarr[i]) < 0; i -= h)
+	      msgarr[i + h] = msgarr[i];
+	    msgarr[i + h] = hold;
 	  }
       }
 }
@@ -453,8 +453,8 @@ swap_message (size_t a, size_t b)
   char *path_a, *path_b;
   char *tmp;
   
-  asprintf (&path_a, "%s/%s", mbox_path, mu_umaxtostr (0, a));
-  asprintf (&path_b, "%s/%s", mbox_path, mu_umaxtostr (1, b));
+  path_a = mh_safe_make_file_name (mbox_path, mu_umaxtostr (0, a));
+  path_b = mh_safe_make_file_name (mbox_path, mu_umaxtostr (1, b));
   tmp = mu_tempname (mbox_path);
   rename (path_a, tmp);
   unlink (path_a);
@@ -469,12 +469,12 @@ transpose(size_t i, size_t n)
 {
   size_t j;
 
-  for (j = i+1; j < msgset.count; j++)
-    if (msgset.list[j] == n)
+  for (j = i+1; j < msgcount; j++)
+    if (msgarr[j] == n)
       {
-	size_t t = msgset.list[i];
-	msgset.list[i] = msgset.list[j];
-	msgset.list[j] = t;
+	size_t t = msgarr[i];
+	msgarr[i] = msgarr[j];
+	msgarr[j] = t;
 	break;
       }
 }
@@ -491,26 +491,25 @@ void
 sort ()
 {
   size_t *oldlist, i;
-  oldlist = xmalloc (msgset.count * sizeof (*oldlist));
-  memcpy (oldlist, msgset.list, msgset.count * sizeof (*oldlist));
+  oldlist = mu_alloc (msgcount * sizeof (*oldlist));
+  memcpy (oldlist, msgarr, msgcount * sizeof (*oldlist));
 
   switch (algorithm)
     {
     case ARG_QUICKSORT:
-      qsort(msgset.list, msgset.count, sizeof(msgset.list[0]),
-	    comp);
+      qsort (msgarr, msgcount, sizeof (msgarr[0]), comp);
       break;
 
     case ARG_SHELL:
-      shell_sort();
+      shell_sort ();
       break;
     }
 
   switch (action)
     {
     case ACTION_LIST:
-      for (i = 0; i < msgset.count; i++)
-	list_message (msgset.list[i]);
+      for (i = 0; i < msgcount; i++)
+	list_message (msgarr[i]);
       break;
 
     default:
@@ -521,27 +520,81 @@ sort ()
       
       if (verbose)
 	fprintf (stderr, _("Transpositions:\n"));
-      for (i = 0, got_signal = 0; !got_signal && i < msgset.count; i++)
+      for (i = 0, got_signal = 0; !got_signal && i < msgcount; i++)
 	{
-	  if (msgset.list[i] != oldlist[i])
+	  if (msgarr[i] != oldlist[i])
 	    {
 	      size_t old_num, new_num;
 	      mu_message_t msg;
 
 	      mu_mailbox_get_message (mbox, oldlist[i], &msg);
 	      mh_message_number (msg, &old_num);
-	      mu_mailbox_get_message (mbox, msgset.list[i], &msg);
+	      mu_mailbox_get_message (mbox, msgarr[i], &msg);
 	      mh_message_number (msg, &new_num);
 	      transpose (i, oldlist[i]);
 	      if (verbose)
-		fprintf (stderr, "{%s, %s}\n",
-			 mu_umaxtostr (0, old_num),
-			 mu_umaxtostr (1, new_num));
+		{
+		  fprintf (stderr, "{%s, %s}",
+			   mu_umaxtostr (0, old_num),
+			   mu_umaxtostr (1, new_num));
+		}
+	      if (old_num == current_num)
+		{
+		  if (verbose)
+		    fputc ('*', stderr);
+		  current_num = new_num;
+		}
+	      else if (new_num == current_num)
+		{
+		  if (verbose)
+		    fputc ('*', stderr);
+		  current_num = old_num;
+		}
+	      if (verbose)
+		fputc ('\n', stderr);
 	      if (action == ACTION_REORDER)
 		swap_message (old_num, new_num);
 	    }
 	}
     }
+  if (action == ACTION_REORDER)
+    {
+      mu_mailbox_close (mbox);
+      mu_mailbox_open (mbox, MU_STREAM_RDWR);
+      mh_mailbox_set_cur (mbox, current_num);
+    }
+}
+
+static int
+_add_msgno (size_t n, void *data)
+{
+  size_t *pidx = data;
+  msgarr[*pidx] = n;
+  ++*pidx;
+  return 0;
+}
+
+static void
+fill_msgarr (mu_msgset_t msgset)
+{
+  size_t i;
+  int rc;
+  
+  rc = mu_msgset_count (msgset, &msgcount);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_msgset_count", NULL, rc);
+      exit (1);
+    }
+      
+  msgarr = mu_calloc (msgcount, sizeof (msgarr[0]));
+  i = 0;
+  rc = mu_msgset_foreach_msgno (msgset, _add_msgno, &i);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_msgset_foreach_msgno", NULL, rc);
+      exit (1);
+    }
 }
 
 
@@ -552,9 +605,10 @@ main (int argc, char **argv)
 {
   int index;
   mu_url_t url;
+  mu_msgset_t msgset;
   
   MU_APP_INIT_NLS ();
-  mh_argp_init (program_version);
+  mh_argp_init ();
   mh_argp_parse (&argc, &argv, 0, options, mh_option,
 		 args_doc, doc, opt_handler, NULL, &index);
   if (!oplist)
@@ -566,7 +620,7 @@ main (int argc, char **argv)
       exit (1);
     }
   
-  mbox = mh_open_folder (mh_current_folder (), 0);
+  mbox = mh_open_folder (mh_current_folder (), MU_STREAM_READ);
   mu_mailbox_get_url (mbox, &url);
   mbox_path = mu_url_to_string (url);
   if (memcmp (mbox_path, "mh:", 3) == 0)
@@ -575,7 +629,14 @@ main (int argc, char **argv)
   argc -= index;
   argv += index;
 
-  mh_msgset_parse (mbox, &msgset, argc, argv, "all");
-  sort (mbox, msgset);
+  mh_mailbox_get_cur (mbox, &current_num);
+
+  mh_msgset_parse (&msgset, mbox, argc, argv, "all");
+  fill_msgarr (msgset);
+  mu_msgset_free (msgset);
+  sort ();
+  mh_global_save_state ();
+  mu_mailbox_destroy (&mbox);
+  
   return 0;
 }
